*****
* Langage     : Clipper 5.0 VF
* Fichier     : Cliptris.prg  
* Author      : Philippe BONNARDEL
*
*              <<< This program is placed in the public domain >>>
*
* Version     : 1.0
* Date        : 08/04/95
* Compil.     : /L/N/W
* Link        : None
* Description : Tetris Game
* Information : Questions, Problems,...? My CIS-ID : 100604,1637
****

#include "box.ch"
#include "inkey.ch"

#define C_VERSION         "1.0"
#define C_YEAR            "1995"

#define WIN_UP            2       
#define WIN_LEFT          17
#define BD_HEIGHT         18
#define BD_WIDTH          20
#define ST_WIDTH          23
#define N_ROW_TEXT        WIN_UP + 2
#define N_COL_TEXT        WIN_LEFT + BD_WIDTH + 3
#define N_LEN_TEXT        12
#define N_COL_SCORE       N_COL_TEXT + N_LEN_TEXT
#define N_LEN_DISP        ST_WIDTH - N_LEN_TEXT - 1
#define N_NEXT_BLOCK_ROW  WIN_UP + 8

// aEnv
#define LEN_ENV           4
#define ENV_SCREEN        1
#define ENV_CURSOR        2
#define ENV_COLOR         3
#define ENV_ROW           4

// aCargo
#define LEN_A_CARGO       16
#define CG_N_GAME          1
#define CG_N_HIGH_SCORE    2
#define CG_N_INIT_SCORE    3 
#define CG_N_SCORE         4
#define CG_N_COUNT_LINES   5
#define CG_N_COUNT_BLOCKS  6 
#define CG_N_LEVEL         7 
#define CG_N_CUR_WAIT      8
#define CG_N_CUR_BLOCK     9 
#define CG_N_NEXT_BLOCK   10
#define CG_N_BLOCK_SIZE   11
#define CG_N_BLOCK_ROW    12 
#define CG_N_BLOCK_COL    13
#define CG_N_RANDOM       14
#define CG_N_INIT_TIK     15
#define CG_N_MAX_BLOCKS   16

// CG_N_GAME
#define N_GAME_OVER        1
#define N_PLAYED           2
#define N_PAUSED           3
#define A_C_STATUS         {"GAME OVER", "PLAY", "PAUSE"}

// Misc.
#define N_MIN_LEVEL        1
#define N_INIT_MAX_BLOCKS  7
#define N_LEVEL_ADD_BLOCKS 7
#define N_PT_PER_BLOCK     1
#define N_PT_PER_LINE      5
#define N_BLOCKS_PER_LEVEL 25
#define N_INIT_WAIT        1100
#define N_TIME_PER_LEVEL   100 
#define C_ELEMENT          ""
#define N_EMPTY            0
#define N_BORDURE          -1
#define C_MASK             "aaaaaaaaa"
#define C_BOARD_CLR        "N"
#define C_FILE_NAME        "CLIPTRIS.INI" 
#define N_LEN_SCORE        5
#define NTRIM(nVal)        LTRIM(STR(nVal))
#define NPADL(nVal)        PADR(NTRIM(nVal), N_LEN_DISP)

// aScore
#define A_STATE {;
  {00, "High Score:", "R/", {|aCargo| NPADL(aCargo[CG_N_HIGH_SCORE])}},;
  {02, "Status    :", "R/", {|aCargo| PADR(A_C_STATUS[aCargo[CG_N_GAME]],;
                               N_LEN_DISP)}},;
  {04, "Score     :", "R+/", {|aCargo| NPADL(aCargo[CG_N_SCORE])}},;
  {11, "Level     :", "GR+/",{|aCargo| NPADL(aCargo[CG_N_LEVEL])}},;
  {13, "Lines     :", "G+/", {|aCargo| NPADL(aCargo[CG_N_COUNT_LINES])}},;
  {14, "Blocks    :", "G+/", {|aCargo| NPADL(aCargo[CG_N_COUNT_BLOCKS])}},;
  {15, "Speed     :", "G+/", {|aCargo| NPADL(INT(N_INIT_WAIT/N_INIT_WAIT) +;
                        INT((N_INIT_WAIT - aCargo[CG_N_CUR_WAIT]) / 100))}}}

// Main
FUNCTION CLIP_TRIS()
LOCAL aEnv, aBlocks, aBoard, aCurBlock, aColor, aCargo
LOCAL nKey, cKey, lAllDrop, lNoExit

Make_Env_Win(@aEnv)
Make_Var(@aBlocks, @aBoard, @aColor, @aCargo)
lAllDrop := .F.
lNoExit  := .T.

DO WHILE lNoExit
  #IFDEF CL50
    BEGINPAINT()
  #ELSE
    DISPBEGIN()
  #ENDIF
  cKey := UPPER(CHR(nKey := INKEY()))
  DO CASE
    CASE nKey == K_ESC
      lNoExit := .F.
    CASE cKey == "N"
      Init_Game(aBlocks, aBoard, @aCurBlock, aCargo)
      Draw_Block(aCurBlock, aColor, aCargo, .T.)
    CASE aCargo[CG_N_GAME] != N_GAME_OVER
      DO CASE 
        CASE aCargo[CG_N_GAME] == N_PAUSED .AND. nKey != 0
          aCargo[CG_N_GAME] := N_PLAYED
        CASE cKey == "P" 
          aCargo[CG_N_GAME] := N_PAUSED
        CASE nKey == K_UP .OR. nKey == K_DOWN
          Draw_Block(aCurBlock, aColor, aCargo, .F.)
          Rotate_Block(@aCurBlock, aCargo, IF(nKey == K_UP, K_LEFT, K_RIGHT))
          IF Hit_Block(aBoard, aCurBlock, aCargo)
             Rotate_Block(@aCurBlock, aCargo, IF(nKey == K_UP, K_RIGHT, K_LEFT))
          ENDIF
          Draw_Block(aCurBlock, aColor, aCargo, .T.)
        CASE nKey == K_LEFT .OR. nKey == K_RIGHT
          Draw_Block(aCurBlock, aColor, aCargo, .F.)
          aCargo[CG_N_BLOCK_COL] += IF(nKey == K_LEFT, -2, 2)
          IF Hit_Block(aBoard, aCurBlock, aCargo)
            aCargo[CG_N_BLOCK_COL] += IF(nKey == K_LEFT, 2, -2)
          ENDIF
          Draw_Block(aCurBlock, aColor, aCargo, .T.)
        CASE nKey == K_BARRE
          lAllDrop := .T.
      ENDCASE
  ENDCASE
  IF aCargo[CG_N_GAME] == N_PLAYED
    IF lAllDrop .OR. Tic_Tac(aCargo[CG_N_CUR_WAIT], aCargo[CG_N_INIT_TIK])
       Draw_Block(aCurBlock, aColor, aCargo, .F.)
       aCargo[CG_N_BLOCK_ROW]++
       IF Hit_Block(aBoard, aCurBlock, aCargo)
        aCargo[CG_N_BLOCK_ROW]--
        Draw_Block(aCurBlock, aColor, aCargo, .T.)
        Place_Block(aBlocks, aBoard, @aCurBlock, aColor, aCargo)
        lAllDrop := .F.
       ENDIF
       Draw_Block(aCurBlock, aColor, aCargo, .T.)
       aCargo[CG_N_INIT_TIK] := SECONDS()
    ENDIF
  ENDIF
  Disp_Score(aBlocks, aColor, aCargo)
  #IFDEF CL50
    ENDPAINT()
  #ELSE
    DISPEND()
  #ENDIF
ENDDO
Rest_Env(aEnv, aCargo)
RETURN (NIL)


STATIC FUNCTION SHADOW (nT, nL, nB, nR)
LOCAL cVerbuff, cHorBuff
RESTSCREEN (nT, nR - 1, nB - 1, nR, ;
      TRANSFORM(cVerbuff := SAVESCREEN (nT, nR - 1, nB - 1, nR), ;
      REPLICATE("X" + CHR(8), LEN(cVerBuff)/2)))
RESTSCREEN (nB, nL, nB, nR, ;
      TRANSFORM(cHorBuff := SAVESCREEN (nB, nL, nB, nR), ;
      REPLICATE("X" + CHR(8), LEN(cHorBuff)/2)))
RETURN (NIL)

*****************************************************************************
*                                                                           *
*                      Env., Win., Var. functions                           *
*                                                                           *
*****************************************************************************

STATIC FUNCTION MAKE_ENV_WIN(aEnv)
aEnv := ARRAY(LEN_ENV)
aEnv[ENV_SCREEN] := SAVESCREEN (0, 0, MAXROW(), MAXCOL())
aEnv[ENV_CURSOR] := SETCURSOR(0)
aEnv[ENV_COLOR]  := SETCOLOR("GR+/W")
aEnv[ENV_ROW]    := ROW()

@ 0, 0 SAY PADC("ClipTris V" + C_VERSION + " - Philippe Bonnardel - " +; 
                C_YEAR, MAXCOL() + 1)

@ MAXROW(), 0 SAY PADC("Exit: Escape  New: N  Pause: P  " +;  
                       CHR(24) + "/" + CHR(25) + ": Rotation  " +;  
                       CHR(27) + "/" + CHR(26) + ": Move  " +;
                       "Space Bar: Drop", MAXCOL() + 1)

RESTSCREEN (1, 0, MAXROW() - 1, MAXCOL(), ;
            REPLICATE (C_MASK, ((MAXCOL() + 1) * (MAXROW() - 1)) / 2))

SETCOLOR("BG/" + C_BOARD_CLR)
@ WIN_UP, WIN_LEFT, WIN_UP + BD_HEIGHT + 1, ;
  WIN_LEFT + BD_WIDTH + ST_WIDTH + 2 BOX B_SINGLE + SPACE(1)

SHADOW(WIN_UP + 1, WIN_LEFT + 2, WIN_UP + BD_HEIGHT + 2, ;
       WIN_LEFT + BD_WIDTH + ST_WIDTH + 4)

@ WIN_UP, WIN_LEFT, WIN_UP + BD_HEIGHT + 1, WIN_LEFT + BD_WIDTH + 1 BOX B_SINGLE
@ WIN_UP, WIN_LEFT + BD_WIDTH + 1 SAY ""
@ WIN_UP + BD_HEIGHT + 1, WIN_LEFT + BD_WIDTH + 1 SAY ""

SETCOLOR("W+/" + C_BOARD_CLR)
AEVAL(A_STATE, {|x| SETPOS(N_ROW_TEXT + x[1], N_COL_TEXT), QQOUT(x[2])})
@ N_NEXT_BLOCK_ROW, N_COL_TEXT SAY "Next      :"
RETURN (NIL)


STATIC FUNCTION REST_ENV(aEnv, aCargo)
LOCAL nHandle
IF aCargo[CG_N_INIT_SCORE] != aCargo[CG_N_HIGH_SCORE]
  nHandle := FCREATE(C_FILE_NAME)
  IF nHandle != - 1
    FWRITE(nHandle, NTRIM(aCargo[CG_N_HIGH_SCORE]))
    FCLOSE(nHandle)
  ENDIF
ENDIF
RESTSCREEN(0, 0, MAXROW(), MAXCOL(), aEnv[ENV_SCREEN])
SETCURSOR(aEnv[ENV_CURSOR])
SETCOLOR(aEnv[ENV_COLOR])
SETPOS(aEnv[ENV_ROW], 0)
RETURN (NIL)


STATIC FUNCTION MAKE_VAR(aBlocks, aBoard, aColor, aCargo)
LOCAL cBuffer, nHandle
aBlocks := {{.F.,.T.,.F.,.F.,;
             .F.,.T.,.F.,.F.,;
             .F.,.T.,.F.,.F.,;
             .F.,.T.,.F.,.F.},;
            {.F.,.T.,.T.,;
             .F.,.T.,.F.,;
             .F.,.T.,.F.},;
            {.T.,.T.,.F.,;
             .F.,.T.,.F.,;
             .F.,.T.,.F.},;
            {.F.,.T.,.F.,;
             .T.,.T.,.T.,;
             .F.,.F.,.F.},;
            {.T.,.T.,.F.,;
             .F.,.T.,.T.,;
             .F.,.F.,.F.},;
            {.F.,.T.,.T.,;
             .T.,.T.,.F.,;
             .F.,.F.,.F.},;
            {.T.,.T.,;
             .T.,.T.},;
            {.F.,.T.,.T.,;
             .F.,.T.,.F.,;
             .F.,.T.,.T.},;
            {.F.,.T.,.T.,;
             .F.,.T.,.F.,;
             .T.,.T.,.F.},;
            {.F.,.T.,.T.,;
             .F.,.T.,.T.,;
             .F.,.T.,.F.},;
            {.F.,.T.,.F.,;
             .F.,.T.,.F.,;
             .F.,.T.,.F.},;
            {.T.,.T.,.F.,;
             .F.,.T.,.T.,;
             .F.,.F.,.T.},;
            {.F.,.T.,.F.,;
             .F.,.T.,.F.,;
             .T.,.T.,.T.},;
            {.F.,.T.,.F.,;
             .T.,.T.,.F.,;
             .F.,.T.,.T.}}

aColor := {"W+", "G+", "GR+", "R+", "BG+", "B+", "RB+", "RB", "B", "BG", "R",;
           "GR", "G", "B"}
aBoard := ARRAY(BD_HEIGHT + 1, BD_WIDTH + 3)
aCargo := ARRAY(LEN_A_CARGO)
aCargo[CG_N_GAME]     := N_GAME_OVER
aCargo[CG_N_RANDOM]   := INT(SECONDS() * 1000)
aCargo[CG_N_CUR_WAIT] := N_INIT_WAIT

Init_Score(aCargo)

aCargo[CG_N_INIT_SCORE] := aCargo[CG_N_HIGH_SCORE] := 0
IF FILE(C_FILE_NAME)
  nHandle := FOPEN(C_FILE_NAME)
  IF nHandle != - 1
    cBuffer := SPACE(N_LEN_SCORE)
    FREAD(nHandle, @cBuffer, N_LEN_SCORE)
    FCLOSE(nHandle)
    aCargo[CG_N_HIGH_SCORE] := VAL(cBuffer)
  ENDIF
ENDIF
RETURN (NIL)


STATIC FUNCTION DISP_SCORE(aBlocks, aColor, aCargo)
AEVAL(A_STATE, {|x| SETCOLOR(x[3]+C_BOARD_CLR), ;
                    SETPOS(N_ROW_TEXT + x[1], N_COL_SCORE), ;
                    QQOUT(EVAL(x[4],aCargo))})
IF aCargo[CG_N_LEVEL] >= N_LEVEL_ADD_BLOCKS
  Draw_Next_Block(aBlocks, aColor, aCargo)
ENDIF
RETURN (NIL)


STATIC FUNCTION INIT_SCORE(aCargo)
aCargo[CG_N_SCORE]        := 0
aCargo[CG_N_COUNT_LINES]  := 0
aCargo[CG_N_COUNT_BLOCKS] := 0
aCargo[CG_N_LEVEL]        := N_MIN_LEVEL
RETURN (NIL)


STATIC FUNCTION RANDOM_BLOCK(aCargo, nMax)
#define IM      7875
#define IA      421
#define IC      1663
aCargo[CG_N_RANDOM]   := (aCargo[CG_N_RANDOM] * IA + IC) % IM
RETURN (INT((aCargo[CG_N_RANDOM] / IM) * nMax) + 1)


FUNCTION TIC_TAC(nXms, nTop)
LOCAL nTic, nTac, nElapsed 
nTac     := INT((nXms / 55) + IF(nXms % 55 > 0, 1, 0))
nElapsed := INT((SECONDS() - nTop) * 1000)
nTic     := INT((IF(nElapsed < 0, nElapsed += 86400000, nElapsed) / 55) + ;
            IF (nElapsed % 55 > 0, 1, 0))
RETURN(IF(nTic >= nTac, .T., .F.))


STATIC FUNCTION INIT_GAME(aBlocks, aBoard, aCurBlock, aCargo)
LOCAL i, j

@ WIN_UP + 1, WIN_LEFT + 1 CLEAR TO WIN_UP + BD_HEIGHT, WIN_LEFT + BD_WIDTH

FOR i := 1 TO BD_HEIGHT
  FOR j := 2 TO BD_WIDTH + 1
    aBoard[i,j] := N_EMPTY
  NEXT
  aBoard[i,1] := aBoard[i,BD_WIDTH+2] := aBoard[i,BD_WIDTH+3] := N_BORDURE
NEXT
AFILL(aBoard[i], N_BORDURE)

aCargo[CG_N_GAME]       := N_PLAYED
aCargo[CG_N_CUR_WAIT]   := N_INIT_WAIT
aCargo[CG_N_MAX_BLOCKS] := N_INIT_MAX_BLOCKS
aCargo[CG_N_NEXT_BLOCK] := Random_Block(aCargo, aCargo[CG_N_MAX_BLOCKS])

Init_Score(aCargo)
Clear_Next_Block()
New_Block(aBlocks, aBoard, @aCurBlock, aCargo)

aCargo[CG_N_INIT_TIK] := SECONDS()
RETURN (NIL)


STATIC FUNCTION CLEAR_NEXT_BLOCK()
@ N_NEXT_BLOCK_ROW, N_COL_SCORE CLEAR TO ;
  N_NEXT_BLOCK_ROW + 3, N_COL_SCORE + 7
RETURN (NIL)


*****************************************************************************
*                                                                           *
*                               Block functions                             *
*                                                                           *
*****************************************************************************

STATIC FUNCTION NEW_BLOCK(aBlocks, aBoard, aCurBlock, aCargo)
LOCAL i
aCargo[CG_N_CUR_BLOCK]  := aCargo[CG_N_NEXT_BLOCK]
aCargo[CG_N_NEXT_BLOCK] := Random_Block(aCargo, aCargo[CG_N_MAX_BLOCKS])

IF ++aCargo[CG_N_COUNT_BLOCKS] % N_BLOCKS_PER_LEVEL == 0 .AND.;
   aCargo[CG_N_CUR_WAIT] > 0 
  IF ++aCargo[CG_N_LEVEL] == N_LEVEL_ADD_BLOCKS
    aCargo[CG_N_CUR_WAIT]   := N_INIT_WAIT - N_TIME_PER_LEVEL
    aCargo[CG_N_MAX_BLOCKS] := LEN(aBlocks)
  ELSE
    aCargo[CG_N_CUR_WAIT]  -= N_TIME_PER_LEVEL
  ENDIF
ENDIF

aCargo[CG_N_SCORE]      += aCargo[CG_N_LEVEL] * N_PT_PER_BLOCK
IF aCargo[CG_N_SCORE] > aCargo[CG_N_HIGH_SCORE]
  aCargo[CG_N_HIGH_SCORE] := aCargo[CG_N_SCORE]
ENDIF

aCargo[CG_N_BLOCK_SIZE] := LEN(aBlocks[aCargo[CG_N_CUR_BLOCK]]) ^ 0.5
aCargo[CG_N_BLOCK_ROW]  := 0
aCargo[CG_N_BLOCK_COL]  := BD_WIDTH / 2
aCurBlock := ACLONE(aBlocks[aCargo[CG_N_CUR_BLOCK]])

IF Hit_Block(aBoard, aCurBlock, aCargo)
  ACOPY(aCurBlock, aCurBlock, aCargo[CG_N_BLOCK_SIZE] + 1)
  AFILL(aCurBlock, .F., LEN(aCurBlock) - aCargo[CG_N_BLOCK_SIZE] + 1)
  DO WHILE Hit_Block(aBoard, aCurBlock, aCargo)
    ACOPY(aCurBlock, aCurBlock, aCargo[CG_N_BLOCK_SIZE] + 1)
  ENDDO
  aCargo[CG_N_GAME] := N_GAME_OVER
ENDIF
RETURN (NIL)


STATIC FUNCTION DRAW_BLOCK(aCurBlock, aColor, aCargo, lDraw)
LOCAL i, j
SETCOLOR(IF(lDraw, aColor[aCargo[CG_N_CUR_BLOCK]], C_BOARD_CLR + "/" + C_BOARD_CLR))
FOR i := 0 TO aCargo[CG_N_BLOCK_SIZE] - 1
  FOR j := 0 TO aCargo[CG_N_BLOCK_SIZE] - 1
    IF aCurBlock[i*aCargo[CG_N_BLOCK_SIZE]+j+1]
      @ aCargo[CG_N_BLOCK_ROW] + WIN_UP + i + 1, ;
        aCargo[CG_N_BLOCK_COL] + WIN_LEFT + j*2 - 1 SAY C_ELEMENT
    ENDIF
  NEXT
NEXT
RETURN (NIL)
       

STATIC FUNCTION DRAW_BCKGND(aBoard, aColor)
LOCAL i, j
FOR i := 1 TO BD_HEIGHT 
  FOR j := 2 TO BD_WIDTH STEP 2
    SETCOLOR(IF (aBoard[i,j] != N_EMPTY, aColor[aBoard[i,j]], C_BOARD_CLR + "/" +C_BOARD_CLR))
    @ WIN_UP + i, WIN_LEFT + j - 1 SAY C_ELEMENT
  NEXT
NEXT
RETURN(NIL)


STATIC FUNCTION DRAW_NEXT_BLOCK(aBlocks, aColor, aCargo)
LOCAL i, j, nSize
SETCOLOR(aColor[aCargo[CG_N_NEXT_BLOCK]])
nSize := LEN(aBlocks[aCargo[CG_N_NEXT_BLOCK]] ) ^ 0.5
Clear_Next_Block()
FOR i := 0 TO nSize - 1
  FOR j := 0 TO nSize - 1
    IF aBlocks[aCargo[CG_N_NEXT_BLOCK],i*nSize+j+1]
      @ N_NEXT_BLOCK_ROW + i, N_COL_SCORE + j*2 SAY C_ELEMENT
    ENDIF
  NEXT
NEXT
RETURN (NIL)

STATIC FUNCTION ROTATE_BLOCK(aCurBlock, aCargo, nSens)
LOCAL i, j, aTmpBlock
DO CASE
  CASE aCargo[CG_N_BLOCK_SIZE] == 4
    aTmpBlock := ARRAY(LEN(aCurBlock))
    IF nSens == K_LEFT
      FOR i := 0 TO 3
        FOR j := 0 TO 3
          aTmpBlock[i*4+j+1] := aCurBlock[j*4+i+1]
        NEXT
      NEXT
    ELSE
      FOR i := 0 TO 3
        FOR j := 0 TO 3
          aTmpBlock[j*4+i+1] := aCurBlock[i*4+j+1]
        NEXT
      NEXT
    ENDIF
    aCurBlock := aTmpBlock
  CASE aCargo[CG_N_BLOCK_SIZE] == 3
    aTmpBlock := ARRAY(LEN(aCurBlock))
    IF nSens == K_LEFT
      FOR i := 0 TO 2
        FOR j := 0 TO 2
          aTmpBlock[(2-j)*3+i+1] := aCurBlock[i*3+j+1]
        NEXT
      NEXT
    ELSE
      FOR i := 0 TO 2
        FOR j := 0 TO 2
          aTmpBlock[i*3+j+1] := aCurBlock[(2-j)*3+i+1]
        NEXT
      NEXT
    ENDIF
    aCurBlock := aTmpBlock
ENDCASE
RETURN (NIL)


STATIC FUNCTION HIT_BLOCK(aBoard, aCurBlock, aCargo)
LOCAL i,j
FOR i := 0 TO aCargo[CG_N_BLOCK_SIZE] - 1
  FOR j := 0 TO aCargo[CG_N_BLOCK_SIZE] - 1
    IF aCurBlock[i*aCargo[CG_N_BLOCK_SIZE]+j+1] .AND.;
       aBoard[aCargo[CG_N_BLOCK_ROW]+i+1,aCargo[CG_N_BLOCK_COL]+j*2+1] != N_EMPTY
         RETURN(.T.)
    ENDIF
  NEXT
NEXT
RETURN (.F.)
       

STATIC FUNCTION PLACE_BLOCK(aBlocks, aBoard, aCurBlock, aColor, aCargo)
LOCAL i,j, nLines
FOR i := 0 TO aCargo[CG_N_BLOCK_SIZE] - 1
  FOR j := 0 TO aCargo[CG_N_BLOCK_SIZE] - 1
    IF aCurBlock[i*aCargo[CG_N_BLOCK_SIZE]+j+1]
      aBoard[aCargo[CG_N_BLOCK_ROW]+i+1,aCargo[CG_N_BLOCK_COL]+j*2] := ; 
      aBoard[aCargo[CG_N_BLOCK_ROW]+i+1,aCargo[CG_N_BLOCK_COL]+j*2+1] := ;
      aCargo[CG_N_CUR_BLOCK]
    ENDIF
  NEXT
NEXT
IF Remove_Lines(aBoard, @nLines)
  TONE(1100,1)
  TONE(1500,1)
  Draw_BckGnd(aBoard, aColor)
  aCargo[CG_N_SCORE] += (aCargo[CG_N_LEVEL] * N_PT_PER_LINE) * nLines
  aCargo[CG_N_COUNT_LINES] += nLines
ELSE
  Draw_Block(aCurBlock, aColor, aCargo, .T.)
ENDIF
New_Block(aBlocks, aBoard, @aCurBlock, aCargo)
RETURN (NIL)


STATIC FUNCTION REMOVE_LINES(aBoard, nLines)
LOCAL i,j,k,l, bRemove
bRemove := .F.
i := BD_HEIGHT
nLines := 0
DO WHILE i >= 1
  FOR j := 2 TO BD_WIDTH + 1
    IF aBoard[i,j] == N_EMPTY
      EXIT
    ENDIF
  NEXT
  IF j > BD_WIDTH + 1
    FOR k := i TO 2 STEP - 1
      FOR l := 2 TO BD_WIDTH + 1
        aBoard[k,l] := aBoard[k-1,l]
      NEXT
      FOR l := 2 TO BD_WIDTH + 1
        aBoard[1,l] := N_EMPTY
      NEXT
    NEXT
    bRemove := .T.
    nLines++
  ELSE
    i--
  ENDIF
ENDDO
RETURN(bRemove)


